/**
 * This callback is called on a socket.on "corpus_analysis_send_meta_data".
 * Handels incoming corpus metadata
 */
function saveMetaData() {
  let [payload, client, results, rest] = arguments;
  results.metaData.init(payload)
  client.recivedMetaData = true;
  console.info('Metada saved:', results.metaData);
}

/**
 * This callback should be registered to the SocketEventListener 'recieveQueryStatus'.
 * It just gets the incoming status data of the issued query
 * and does some preperation work like hiding or showing elements and deleting
 * the data from the last query.
 */
function prepareQueryData() {
  // deletes old data from query issued before this new query
  let [payload, client, results, rest] = arguments;
  results.init();
  client.requestQueryProgress = 0;

}

function saveQueryData(args) {
  let [payload, client, results, rest] = arguments;
  results.data.addData(payload);
  client.requestQueryProgress = payload.progress;
  console.info('Query data chunk saved', results.data);
}

function querySetup(payload, client) {
  // deletes old data from query issued before this new query
  client.results.clearAll();
  // load necessary HTMLElements with selectory syntax and save them as fields
  client.getHTMLElements(['#query-progress-bar', '#query-results-user-feedback',
                          '#recieved-match-count', '#total-match-count',
                          '#text-lookup-count', '#text-lookup-titles',
                          '#query-results-create', '#add-to-sub-results']);
  client.recievedMatchCount.textContent = 0;
  client.totalMatchCount.textContent = `${payload.match_count}`;
  client.queryResultsUserFeedback.classList.toggle('hide');
  client.queryProgressBar.classList.toggle('hide');
  client.queryProgressBar.lastElementChild.style.width = '0%';
  if (client.dynamicMode) {
    client.addToSubResults.toggleAttribute('disabled');
    client.queryResultsCreate.classList.toggle('disabled');
  }
}

/**
 * This callback should be registered to the SocketEventListener 'recieveQueryData'
 * It takes the incoming chunk and renders the results using the
 * results.jsList object. It can either handle live incoming data chunks or
 * already loaded/imported results data.
 */
function queryRenderResults(payload, client) {
  client.getHTMLElements(['#recieved-match-count', '#match-count',
                          '#display-options-form-expert_mode']);
  const renderResults = (data) => {
    /**
     * resultItem saves the incoming chunk matches as objects to add those later
     * to the client.results.jsList
     */
   let resultItems = [];
   // get infos for full match row
   for (let [index, match] of data.matches.entries()) {
     resultItems.push({...match, ...{'index': index + client.results.data.matches.length}});
   }
     client.results.jsList.add(resultItems, (items) => {
       for (let item of items) {
         item.elm = client.results.jsList.createResultRowElement(item, data);
       }
     });
  }
  if (client.dynamicMode) {
    if (payload.chunk.cpos_ranges == true) {
      client.results.data['cpos_ranges'] = true;
    } else {
      client.results.data['cpos_ranges'] = false;
    }
    renderResults(payload.chunk);
    helperQueryRenderResults(payload, client);
    console.info('Result progress is:', client.requestQueryProgress);
    if (client.requestQueryProgress === 100) {
      /**
       * activate, hide or show elements if all reults have been recieved
       * also load some new elements taht have not ben loaded before
       */
      client.queryProgressBar.classList.toggle('hide');
      client.queryResultsUserFeedback.classList.toggle('hide');
      client.queryResultsCreate.classList.toggle('disabled');
      client.addToSubResults.toggleAttribute('disabled');
    //   addToSubResultsElement.removeAttribute("disabled");
    //   // inital expert mode check and sub results activation
    //   client.results.jsList.activateInspect();
    //   if (addToSubResultsElement.checked) {
    //     client.results.jsList.activateAddToSubResults();
    //   }
    //   if (expertModeSwitchElement.checked) {
    //     client.results.jsList.expertModeOn("query-display");
    //   }
    }
  } else if (!client.dynamicMode) {
    renderResults(payload);
    helperQueryRenderResults({'chunk': payload}, client);
    client.queryProgressBar.classList.toggle('hide');
    client.queryResultsUserFeedback.classList.toggle('hide');
    client.results.jsList.activateInspect();
    if (client.displayOptionsFormExpertMode.checked) {
      client.results.jsList.expertModeOn("query-display");
    }
  }
}

/**
 * Helper function that saves result data into the client.results.data object.
 * Also does some showing and hiding of Elements and user feedback text.
 */
function helperQueryRenderResults (payload, client) {
  // updating table on finished item creation callback via createResultRowElement
  client.results.jsList.update();
  client.results.jsList.changeContext(); // sets lr context on first result load
  // incorporating new chunk results into full results
  client.results.data.matches.push(...payload.chunk.matches);
  client.results.data.addData(payload.chunk.cpos_lookup, 'cpos_lookup');
  client.results.data.addData(payload.chunk.text_lookup, 'text_lookup');
  // complete metaData
  // client.results.metaData.add();
  // show user current and total match count
  client.recievedMatchCount.textContent = `${client.results.data.matches.length}`;
  client.textLookupCount.textContent = `${Object.keys(client.results.data.text_lookup).length}`;
  let titles = new Array();
  for (let [key, value] of Object.entries(client.results.data.text_lookup)) {
    titles.push(`${value.title} (${value.publishing_year})`);
  };
  client.textLookupTitles.textContent = `${titles.join(", ")}`;
  // update progress bar and requestQueryProgress
  client.queryProgressBar.lastElementChild.style.width =  `${payload.progress}%`;
  client.requestQueryProgress = payload.progress;
}

// export callbacks
export { prepareQueryData, saveMetaData, saveQueryData };